// used to compile            
//      g++ -o flowmeters flowmeters.cpp -lwiringPi


#include </var/lib/reef-pi/flowmeters/flowmeters_counters.cpp>

int main()
{
    readSettingsFromFile();  // read settings from text file
    setupFlowPins();  // setup Pi pins
   
    while(1)  // Main program loop
    {
        for (short a=0; a < totalFlowMeters; a++)  // read each flow meter
        {
            if (millis() - oldTime[a] > 1000)  // read flow meter every second
            {
                flowRate[a] = ((1000.0 / (millis() - oldTime[a])) * pulseCount[a]) / calibrationFactor;  // flow rate in liters per minute
                float volume;
                if (millis() - oldTime[a] < 1000) {volume = (flowRate[a] / 60) * (1000/(millis() - oldTime[a]));}  // liters pumped since last check
                else volume = (flowRate[a] / 60.0) * ((millis() - oldTime[a])/1000.0);  // liters pumped since last check
                
                if (calRun[a] == 0)  // if not calibrating
                {
                    if (isnan(totalFlowS[a])) {totalFlowS[a] = 0.0;}  // check if total flow is a number
                    totalFlowS[a] = totalFlowS[a] + volume;  // add liters pumped since last check to the total flow - this value is liters per minute and saved to file
                
                    // convert flow rate and total flow to match measurement system and measurement type
                    switch (measSystem)  // measurement system
                    {
                        case 1:  // US Systems
                            switch (measType)  // measurement type
                            {
                                case 1: flowRate[a] = flowRate[a] * 2.113; totalFlow[a] = (float)totalFlowS[a] * 2.113; break;  // US pints per minute
                                case 2: flowRate[a] = flowRate[a] * 1.05669; totalFlow[a] = (float)totalFlowS[a] * 1.05669; break;  // US quarts per minute
                                case 3: totalFlow[a] = totalFlowS[a]; break;  // liters per minute
                                case 4: flowRate[a] = flowRate[a] / 3.785; totalFlow[a] = (float)totalFlowS[a] / 3.785; break;  // US gallons per minute
                            }
                        break;
                        case 2:  // Imperial
                            switch (measType)  // measurement type
                            {
                                case 1: flowRate[a] = flowRate[a] * 1.76; totalFlow[a] = (float)totalFlowS[a] * 1.76; break;  // Imperial pints per minute
                                case 2: flowRate[a] = flowRate[a] / 1.137; totalFlow[a] = (float)totalFlowS[a] * 1.137; break;  // Imperial quarts per minute
                                case 3: totalFlow[a] = totalFlowS[a]; break;  // liters per minute
                                case 4: flowRate[a] = flowRate[a] / 4.546; totalFlow[a] = (float)totalFlowS[a] / 4.546; break;  // Imperial gallons per minute
                            }
                        break;
                    }
                // std::cout << "totalFlow[a]          " << totalFlow[a] << '\n';
                }
                else if (calRun[a] == 1)  // get pulses for calibration 
                {
                    if (calType == 0)
                    {
                        calPulses[a] = calPulses[a] + pulseCount[a];
                        calCount++;
                    }
                    else if (calType == 1)
                    {
                        calPulses[a] = pulseCount[a];
                    }
                }
                
                // convert flow rate to time format
                switch (viewFormat)  // viewing format
                {
                    case 1: flowRate[a] = flowRate[a] / 60; break;  // per second
                    case 3: flowRate[a] = flowRate[a] * 60; break;  // per hour
                    case 4: flowRate[a] = flowRate[a] * 1440; break;  // per day
                }  

                std::string filename = "/var/lib/reef-pi/flowmeters/totalFlow_" + to_string(a) + ".txt";  // construct file name to save total flow for meter
                // std::cout << filename << std::endl;
                
                fstream newfile;
                newfile.open(filename,ios::out);  // open file to save total flow
                if(newfile.is_open())  // checking if the file is open
                {
                    newfile << totalFlow[a];  // save total flow
                    newfile.close();  // close file
                }

                filename = "/var/lib/reef-pi/flowmeters/totalFlowS_" + to_string(a) + ".txt";  // construct file name to save flow rate for meter 
                // std::cout << filename << std::endl;
                
                newfile.open(filename,ios::out);  // open file to save flow rate
                if(newfile.is_open())  // checking if the file is open
                {
                    newfile << totalFlowS[a];  // save flow rate
                    newfile.close();  // close file
                }

                filename = "/var/lib/reef-pi/flowmeters/flowRate_" + to_string(a) + ".txt";  // construct file name to save flow rate for meter 
                // std::cout << filename << std::endl;
                
                newfile.open(filename,ios::out);  // open file to save flow rate
                if(newfile.is_open())  // checking if the file is open
                {
                    newfile << flowRate[a];  // save flow rate
                    newfile.close();  // close file
                }

                if (debug == 1)
                {
                    std::cout << pulseCount[a] << std::endl;
                    std::cout << flowRate[a] << std::endl;
                    std::cout << totalFlow[a] << std::endl;
                    std::cout << "----------------------------------" << std::endl;
                }

                pulseCount[a] = 0;
                oldTime[a] = millis();
	            // cout << "----------------------------------";
                // std::cout << a << std::endl;
            }
        }
        sleep(1);
    }
    return 0;
}