// used to compile            
//      g++ -o flowmeters flowmeters.cpp -lwiringPi

#include <unistd.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include <string>
#include <wiringPi.h>

using namespace std;

float totalFlowMeters = 0;  // track total flow meters connected
unsigned long oldTime[10] = {0};  // track reading intervals
volatile int pulseCount[10] = {0};  // track number of pulses counted
float flowRate[10];  // track flow rate
float calibrationFactor = 4.5;  // flow sensor pulses per litre per minute - (F number on flow sensor)
float totalFlow[10] = {0};  // total flow converted to measurement type selected
float totalFlowS[10] = {0};  // total flow - saved in liters
int flowWiringPiPin[10];  // wiringPi pin numbers
int flow_GPIO[10];  // GPIO pin numbers
short calRun[10] = {0};  // track if user is calibrating
volatile int calPulses[10] = {0};  // number of pulses during calibration
volatile int calCount = 0;  // used to count how many times sensor is read during calibration
short calType = 0;  // track which calibration type is being used
short measSystem = 1;  // track if using imperial or US measuring system
short measType = 3;  // track if using pints, quarts, litres or gallons
short viewFormat = 3;  // track if using seconds, minutes, days or hours
short debug = 0;  // track if data should be printed to SSH terminal

// interrupt counters
void flow0Interrupt(void) {pulseCount[0]++;}  // flow meter 1
void flow1Interrupt(void) {pulseCount[1]++;}  // flow meter 2
void flow2Interrupt(void) {pulseCount[2]++;}  // flow meter 3
void flow3Interrupt(void) {pulseCount[3]++;}  // flow meter 4
void flow4Interrupt(void) {pulseCount[4]++;}  // flow meter 5
void flow5Interrupt(void) {pulseCount[5]++;}  // flow meter 6
void flow6Interrupt(void) {pulseCount[6]++;}  // flow meter 7
void flow7Interrupt(void) {pulseCount[7]++;}  // flow meter 8
void flow8Interrupt(void) {pulseCount[8]++;}  // flow meter 9
void flow9Interrupt(void) {pulseCount[9]++;}  // flow meter 10

void setupFlowPins() {
	// cout << "setupFlowPins" << endl;

    for (short a=0; a < totalFlowMeters; a++)  // assign wiringPi pin number based on GPIO number
    {
        switch (flow_GPIO[a])
        {
            case 5: flowWiringPiPin[a] = 21; break;
            case 6: flowWiringPiPin[a] = 22; break;
            case 7: flowWiringPiPin[a] = 11; break;
            case 8: flowWiringPiPin[a] = 10; break;
            case 9: flowWiringPiPin[a] = 13; break;
            case 10: flowWiringPiPin[a] = 12; break;
            case 11: flowWiringPiPin[a] = 14; break;
            case 12: flowWiringPiPin[a] = 26; break;
            case 13: flowWiringPiPin[a] = 23; break;
            case 14: flowWiringPiPin[a] = 15; break;
            case 15: flowWiringPiPin[a] = 16; break;
            case 16: flowWiringPiPin[a] = 27; break;
            case 17: flowWiringPiPin[a] = 0; break;
            case 18: flowWiringPiPin[a] = 1; break;
            case 19: flowWiringPiPin[a] = 24; break;
            case 20: flowWiringPiPin[a] = 28; break;
            case 21: flowWiringPiPin[a] = 29; break;
            case 22: flowWiringPiPin[a] = 3; break;
            case 23: flowWiringPiPin[a] = 4; break;
            case 24: flowWiringPiPin[a] = 5; break;
            case 25: flowWiringPiPin[a] = 6; break;
            case 26: flowWiringPiPin[a] = 25; break;
            case 27: flowWiringPiPin[a] = 2; break;
        }
    }

    wiringPiSetup(); // wiringPi library startup

    if (totalFlowMeters > 0) {
        pinMode(flowWiringPiPin[0], INPUT);  // configure Pi pin as input
        wiringPiISR (flowWiringPiPin[0], INT_EDGE_FALLING, flow0Interrupt);  // enable interrupt on Pi pin
    }
    if (totalFlowMeters > 1) {
        pinMode(flowWiringPiPin[1], INPUT);  // configure Pi pin as input
        wiringPiISR (flowWiringPiPin[1], INT_EDGE_FALLING, flow1Interrupt);  // enable interrupt on Pi pin
    }
    if (totalFlowMeters > 2) {
        pinMode(flowWiringPiPin[2], INPUT);  // configure Pi pin as input
        wiringPiISR (flowWiringPiPin[2], INT_EDGE_FALLING, flow2Interrupt);  // enable interrupt on Pi pin
    }
    if (totalFlowMeters > 3) {
        pinMode(flowWiringPiPin[3], INPUT);  // configure Pi pin as input
        wiringPiISR (flowWiringPiPin[3], INT_EDGE_FALLING, flow3Interrupt);  // enable interrupt on Pi pin
    }
    if (totalFlowMeters > 4) {
        pinMode(flowWiringPiPin[4], INPUT);  // configure Pi pin as input
        wiringPiISR (flowWiringPiPin[4], INT_EDGE_FALLING, flow4Interrupt);  // enable interrupt on Pi pin
    }
    if (totalFlowMeters > 5) {
        pinMode(flowWiringPiPin[5], INPUT);  // configure Pi pin as input
        wiringPiISR (flowWiringPiPin[5], INT_EDGE_FALLING, flow5Interrupt);  // enable interrupt on Pi pin
    }
    if (totalFlowMeters > 6) {
        pinMode(flowWiringPiPin[6], INPUT);  // configure Pi pin as input
        wiringPiISR (flowWiringPiPin[6], INT_EDGE_FALLING, flow6Interrupt);  // enable interrupt on Pi pin
    }
    if (totalFlowMeters > 7) {
        pinMode(flowWiringPiPin[7], INPUT);  // configure Pi pin as input
        wiringPiISR (flowWiringPiPin[7], INT_EDGE_FALLING, flow7Interrupt);  // enable interrupt on Pi pin
    }
    if (totalFlowMeters > 8) {
        pinMode(flowWiringPiPin[8], INPUT);  // configure Pi pin as input
        wiringPiISR (flowWiringPiPin[8], INT_EDGE_FALLING, flow8Interrupt);  // enable interrupt on Pi pin
    }
    if (totalFlowMeters > 9) {
        pinMode(flowWiringPiPin[9], INPUT);  // configure Pi pin as input
        wiringPiISR (flowWiringPiPin[9], INT_EDGE_FALLING, flow9Interrupt);  // enable interrupt on Pi pin
    }
}

void readSettingsFromFile() {
    std::string settings[16];  // temporary storage for settings from text file

    fstream newfile;
    newfile.open("settings.txt",ios::in);  // open settings file
    if (newfile.is_open()) {  // check if the file is open
        std::string lineFile;  // use to store line from file
        short a = 0;  // count through lines
        std::string find = "= '";  // line in text file must contain this substring to be considered a setting

        while(getline(newfile, lineFile)) {  // read line by line from file
            if (lineFile.find(find) != std::string::npos) {  // search line to see if it's a setting
                std::size_t pos = lineFile.find("'") + 1;  // get start location of setting in line
                std::size_t count = lineFile.find_last_of(" = '");  // get end location of setting in line                
                count = count - pos;  // calculate how many characters setting is so we can extract from line

                settings[a] = lineFile.substr (pos,count);  // extract setting value to a temporary array
                // std::cout << "settings[a]        " << settings[a] << '\n';
                a++;
            }
        }
        newfile.close();  // close file
        
        short b = 0;  // used to navigate through GPIO pin array
        for (short a=0; a < 16; a++)  // fill program variables with settings extracted from text file
        {
            if (a == 0) {totalFlowMeters = std::stoi(settings[a]);}
            else if (a == 11) {calibrationFactor = std::stof(settings[a]);}
            else if (a == 12) {measSystem = std::stoi(settings[a]);}
            else if (a == 13) {measType = std::stoi(settings[a]);}
            else if (a == 14) {viewFormat = std::stoi(settings[a]);}
            else if (a == 15) {debug = std::stoi(settings[a]);}
            else {flow_GPIO[b] = std::stoi(settings[a]); b++;}
        }
    }
    
    for (short a=0; a < totalFlowMeters; a++)  // read total flow from text file for each meter
    {
        fstream newfile;
        std::string filename = "/var/lib/reef-pi/flowmeters/totalFlowS_" + to_string(a) + ".txt";  // construct file name for total flow
        // std::cout << filename << std::endl;

        newfile.open(filename,ios::in);  // open file to read total flow
        if (newfile.is_open()){  // check if the file is open
            string lineFile;
            while(getline(newfile, lineFile)){  // read data from file line by line, file contains only one line
                try {totalFlowS[a] = std::stof(lineFile);}  // load total flow into program variable
                catch(invalid_argument) {std::cout << "Total Flow file is corrupted, total flow will start from 0" << std::endl;}
                // std::cout << "totalFlowS[a]          " << totalFlowS[a] << '\n';
            }
            newfile.close();  // close file
        }
    }


    std::cout << "totalFlowMeters   = " << totalFlowMeters << '\n';
    for (short a=0; a < totalFlowMeters; a++) 
    {
        std::cout << "flow_GPIO         = " << flow_GPIO[a] << '\n';
    }
    std::cout << "calibrationFactor = " << calibrationFactor << '\n';
    std::cout << "measSystem        = " << measSystem << '\n';
    std::cout << "measType          = " << measType << '\n';
    std::cout << "viewFormat        = " << viewFormat << '\n';
    std::cout << "debug             = " << debug << '\n';
}